package models

import (
	"strings"

	"github.com/juju/errors"
)

// ApiTypeName
type ApiTypeName string

const (
	ApiTypeName_GIN_GONIC   ApiTypeName = "Gin Gonic"
	ApiTypeName_GO_SWAGGER  ApiTypeName = "Go Swagger"
	ApiTypeName_MOJOLICIOUS ApiTypeName = "Mojolicious"
	ApiTypeName_NULL        ApiTypeName = ""
)

// GetListOfApiTypeName returns a list of ApiTypeName
func GetListOfApiTypeName() []ApiTypeName {
	return []ApiTypeName{
		ApiTypeName_GIN_GONIC,
		ApiTypeName_GO_SWAGGER,
		ApiTypeName_MOJOLICIOUS,
	}
}

// IsValid validates enum values
func (e ApiTypeName) IsValid() bool {
	for _, v := range GetListOfApiTypeName() {
		if e == v {
			return true
		}
	}

	return false
}

func NewApiTypeName(in string) (ApiTypeName, error) {
	out := ApiTypeName_NULL
	if in != "" {
		out = ApiTypeName(in)
		if !out.IsValid() {
			return ApiTypeName_NULL, errors.BadRequestf("Value %s invalid for enum ApiTypeName", in)
		}
	}
	return out, nil
}

// NewApiTypeNameFromInput
func NewApiTypeNameFromInput(in string) (ApiTypeName, error) {

	in = strings.ToLower(in)
	in = strings.ReplaceAll(in, " ", "-")
	in = strings.ReplaceAll(in, "_", "-")

	out := ApiTypeName_NULL
	if in != "" {
		for _, apiTypeName := range GetListOfApiTypeName() {
			apiTypeStr := strings.ToLower(string(apiTypeName))
			apiTypeStr = strings.ReplaceAll(apiTypeStr, " ", "-")
			apiTypeStr = strings.ReplaceAll(apiTypeStr, "_", "-")
			if in == apiTypeStr {
				return apiTypeName, nil
			}
		}
		return ApiTypeName_NULL, errors.BadRequestf("Value %s invalid for enum ApiTypeName", in)
	}
	return out, nil
}
